package sample;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Random;

import rescuecore2.log.Logger;
import rescuecore2.worldmodel.EntityID;
import sample.com.AssignMessage;
import sample.com.BaseMessage;

public class DSA implements DecentralAssignment 
{
    private EntityID currentTarget;
    private EntityID identity;
    private HashMap<EntityID, EntityID> agentTargetMap = new HashMap<EntityID, EntityID>(); // maps agents to targets
    private HashMap<EntityID, Integer> conflictMap = new HashMap<EntityID, Integer> (); // maps for each target number conflicts
    private HashMap<EntityID, Double> utilityMap = new HashMap<EntityID, Double> (); // maps for each target the utility
    boolean valueHasChanged = true;
    Random rand = new Random();
    int terminationCount = 0;
    
    private boolean DEBUG = false;


    @Override
    public void init(double [][] U, ArrayList<EntityID> agents, ArrayList<EntityID> targets, EntityID identity) 
    {
	agentTargetMap.clear();
	conflictMap.clear();
	utilityMap.clear();
	terminationCount = Params.DSA_NUNBER_CYCLES;
	this.identity = identity;

	// Select a random target
	int rnd = (int) (rand.nextDouble() * (double) targets.size()-1);
	currentTarget = targets.get(rnd); 
	int a = -1;
	for (int i=0; i<agents.size(); i++) {
	    if (agents.get(i) == identity)
		a = i;
	}
	if (a<0) {
	    Logger.debugColor("ERROR during init DSA!",Logger.BG_RED);
	    return; 
	}
	if (DEBUG)
	    Logger.debugColor("agent:" + identity + " target:" + currentTarget,Logger.FG_GREEN);	

	// Copy utilities
	for (int i=0; i<targets.size(); i++) {
	    EntityID t = targets.get(i);
	    int row = a;
	    int col = i;
	    Double u = new Double(U[row][col]);
	    utilityMap.put(t, u);	    
	}
	valueHasChanged = true;
    }

    @Override
    public BaseMessage sendMessage() {
	if (DEBUG)
	    System.out.println("SND Agent " + identity);
	// Send out assigned target when changed
	AssignMessage outMsg = null;
	if (valueHasChanged) {
	    outMsg = new  AssignMessage(identity, currentTarget);
	    valueHasChanged = false;	    
	}
	return outMsg;
    }

    @Override
    public void receiveMessages(ArrayList<BaseMessage> msgs) {
	if (DEBUG)
	    System.out.println("RCV Agent " + identity + "msg size: "+msgs.size());
	// Process neighbors' messages, i.e., update agentTargetMap
	for (int i=0; i<msgs.size(); i++) {
	    BaseMessage msg = msgs.get(i);
	    if (msg.type() == BaseMessage.MSG_ASSIGNMENT) {
		AssignMessage amsg = (AssignMessage) msg;
		EntityID agent = amsg.sender; 
		EntityID target = amsg.target;
		agentTargetMap.put(agent, target);
	    }
	}
	//System.out.println("AgentTargetMap:");
	//System.out.println(agentTargetMap);

	// Update number of conflicts
	conflictMap.clear();
	for (EntityID next : agentTargetMap.values()) {
	    Integer value = conflictMap.get(next);
	    if (value==null) value = new Integer(1);
	    else value++;	    
	    conflictMap.put(next, value);	
	}
	//System.out.println("Conflict Map: ");
	//System.out.println(conflictMap);
    }
    
    @Override
    public boolean compute() {
	if (terminationCount == 0) 
	    return true;

	// Roll the dice
	double p = rand.nextDouble();

	// Go through list of all targets
	for (EntityID next : utilityMap.keySet()) {
	    //EntityID target = next;
	    Double currentTargetValue = utilityMap.get(currentTarget);
	    Double utility = utilityMap.get(next);
	    if (utility <= currentTargetValue)
		continue;
	    int currConf = 0;
	    if (conflictMap.get(currentTarget) != null)
		currConf = conflictMap.get(currentTarget);
	    int nextConf = 0;
	    if (conflictMap.get(next) != null)
		nextConf = conflictMap.get(next);	    
	    int delta = currConf - nextConf;
	    boolean C = currConf > 0;
	    if (delta < 0)
		continue;	    

	    boolean change = false;
	    switch (Params.DSA_VARIANT) {
	    case Params.DSA_A:
		if (delta>0 && p <= Params.DSA_CHANGE_VALUE_PROBABILITY)
		    change=true;
		break;
	    case Params.DSA_B:
		if (delta>0) { 
		    if (p <= Params.DSA_CHANGE_VALUE_PROBABILITY)
		    	change=true;
		}
		else if (delta == 0 && C) { 
		    if (C && p <= Params.DSA_CHANGE_VALUE_PROBABILITY) 
			change=true;
		}
		else if (delta == 0 && !C) { 
		}
		break;
	    default:
		System.err.println("Wrong DSA variant: " + Params.DSA_VARIANT);
		return true; // terminate
	    }
	    if (change) {
		currentTarget = next; 
		valueHasChanged = true;
	    }
	}
	terminationCount--;
	return false;
    }

    @Override
    public EntityID getSelectedTarget() {
	return currentTarget;
    }
    @Override
    public EntityID getIdentity() {
	return identity;
    }    
}
