package sample.efpo;

import java.util.*;

public class EFPO_2 {
	
	int[][] U;
	double[] alphas;
	double[] prices;
	double[] M;
	double[] M2;
	boolean[][] edges;
	boolean[][] edges2;
	double PJ_plus,PJ_minus;
	int[] types;
	double[][] maxReductions;
	
	public EFPO_2( int[][] u)
	{
		U = copyIntSquareArr(u);
	}
	
	public double[][] getAlloc_heu()
	{
		double[][] alloc = getAlloc();
		while(canReduce())
		{
			alloc = getAlloc();
		}
		return alloc;
	}
	
	public double[][] getAlloc()
	{
		double minCutVal = 0;
		alphas = getAlphas();
		prices = getInitPrices();
		Flow_net_2 FN = null;
		int count = 0;
		while( minCutVal < 0.999999)
		{
			//System.out.println("minCutVal: " + minCutVal);
			
			//System.out.print("prices: ");
			//hello.printDoubleArray(prices);
			M = getMArr(prices);
			M2 = getMArr2(prices);
			//System.out.print("M: ");
			//hello.printDoubleArray(M);
			edges = getEdges(prices, M);
			edges2 = getEdges(prices, M2);
			
			FN = new Flow_net_2(alphas,prices,edges,edges2);
			minCutVal = FN.getMinCutValue();
			Vector<Integer> S_plus = FN.getS_plus();
			Vector<Integer> S_zero = FN.getS_zero();
			Vector<Integer> S_minus = FN.getS_minus();
			Vector<Integer> J_plus = FN.getJ_plus();
			Vector<Integer> J_zero = FN.getJ_zero();
			Vector<Integer> J_minus = FN.getJ_minus();
			
			//System.out.println("S_plus: " + S_plus);
			//System.out.println("S_zero: " + S_zero);
			//System.out.println("S_minus: " + S_minus);
			PJ_plus = getPJ(J_plus);
			//System.out.println("J_plus: " + J_plus);
			//System.out.println("J_zero: " + J_zero);
			PJ_minus = getPJ(J_minus);
			//System.out.println("J_minus: " + J_minus);
			//System.out.println("PJ_plus: " + PJ_plus);
			//System.out.println("PJ_minus: " + PJ_minus);
			
			double epsilon1 = 100;
			epsilon1 = Math.min(epsilon1, getEpsilon1_plus_zero(S_plus,J_zero));
			epsilon1 = Math.min(epsilon1, getEpsilon1_zero_minus(S_zero,J_minus));
			epsilon1 = Math.min(epsilon1, getEpsilon1_plus_minus(S_plus,J_minus));
			
			double epsilon2 = getEpsilon2(S_minus,J_minus);
			
			double epsilon = Math.min(epsilon1, epsilon2);
			//System.out.println("epsilon1: " + epsilon1);
			//System.out.println("epsilon2: " + epsilon2);
			//System.out.println("epsilon: " + epsilon);
			
			updateJ_plus(J_plus,epsilon);
			updateJ_minus(J_minus,epsilon);
			
			
			//hello.printDoubleArray(alphas);
			
			//hello.printDoubleArray(M);
			//hello.printBooleanDoubleArray(edges);
			//System.out.print("epsilon1: " + epsilon1);
			if (count == 200)
			  System.exit(0);
			else
				count++;
		}
		
		maxReductions = getMaxReductions(FN);
		System.out.println("max reductions:");
		hello.printDoubleDoubleArray(maxReductions);
		
		types = getTypes(FN);
		System.out.print("types: ");
		hello.printArray(types);
		
		return FN.getAlloc();
	}
	
	
	private void updateJ_plus(Vector<Integer> J_plus, double epsilon)
	{
		Iterator<Integer> iter = J_plus.iterator();
		while(iter.hasNext())
		{
			//System.out.println("updating j plus");
			int j = iter.next();
			//System.out.println("j plus = " + j);
			//System.out.println("before update prices[j] = " + prices[j]);
			prices[j] = prices[j] * (1 + PJ_minus*epsilon);
			//System.out.println("after update prices[j] = " + prices[j]);
		}
	}
	
	private void updateJ_minus(Vector<Integer> J_minus, double epsilon)
	{
		Iterator<Integer> iter = J_minus.iterator();
		while(iter.hasNext())
		{
			int j = iter.next();
			prices[j] = prices[j] * (1 - PJ_plus*epsilon);
		}
	}
	
	private double getEpsilon1_plus_zero(Vector<Integer> S_plus, Vector<Integer>J_zero)
	{
		double ans = 100;
		Iterator<Integer> iterS = S_plus.iterator();
		while(iterS.hasNext())
		{
			Iterator<Integer> iterJ = J_zero.iterator();
			int s = iterS.next(); 
			while(iterJ.hasNext())
			{
				int j = iterJ.next();
				if(!edges[s][j])
				{
					double eps = Math.pow(PJ_minus, -1) * ((M[s]*prices[j]/U[s][j])-1);
					ans = Math.min(ans, eps);
				}
			}
		}
		//System.out.println("epsilon1 +0: " + ans);
		return ans;
	}
	
	private double getEpsilon1_zero_minus(Vector<Integer> S_zero, Vector<Integer>J_minus)
	{
		double ans = 100;
		Iterator<Integer> iterS = S_zero.iterator();
		while(iterS.hasNext())
		{
			Iterator<Integer> iterJ = J_minus.iterator();
			int s = iterS.next(); 
			while(iterJ.hasNext())
			{
				int j = iterJ.next();
			//	System.out.println("epsilon1 0- U[s][j]: " + edges[s][j]);
				if(!edges[s][j])
				{
					double eps = Math.pow(PJ_plus, -1) * (1- U[s][j]/(M[s]*prices[j]));
				//	System.out.println("eps = " + eps);
					ans = Math.min(ans, eps);
				}
			}
		}
		//System.out.println("epsilon1 0-: " + ans);
		return ans;
	}
	
	private double getEpsilon1_plus_minus(Vector<Integer> S_plus, Vector<Integer>J_minus)
	{
		//System.out.println("starting ");
		double ans = 100;
		Iterator<Integer> iterS = S_plus.iterator();
		while(iterS.hasNext())
		{
			Iterator<Integer> iterJ = J_minus.iterator();
			int s = iterS.next(); 
			while(iterJ.hasNext())
			{
				int j = iterJ.next();
				//System.out.println("s = " + s + " j = " + j + " (U[s][j]*PJ_minus + M[s]*prices[j]*PJ_plus) = " + (U[s][j]*PJ_minus + M[s]*prices[j]*PJ_plus));
				if(!edges[s][j])
				{
					double eps =  (M[s]*prices[j]-U[s][j])/(U[s][j]*PJ_minus + M[s]*prices[j]*PJ_plus);
					//System.out.println("epsilon1 +-: " + eps);
					ans = Math.min(ans, eps);
				}
			}
		}
		//System.out.println("ans +-: " + ans);
		return ans;
	}
	
	private double getEpsilon2(Vector<Integer> S_minus,Vector<Integer> J_minus)
	{
		double ans = 100;
		Vector<Vector<Integer>> subsets = getSubsets(S_minus);
		//System.out.println("subsets: " + subsets);
		Iterator<Vector<Integer>> iter = subsets.iterator();
		while(iter.hasNext())
		{
			Vector t = iter.next();
			if(t.size() > 0)
			{
			
				double alphaT = getSetAlphas(t);
				Vector<Integer> jT = getJset(t,J_minus);
				double pJT = getSetPrices(jT);
				//System.out.println("t: " + t);
				//System.out.println("alphaT: " + alphaT);
				//System.out.println("jT: " + jT);
				//System.out.println("pJT: " + pJT);
				
				double eps2 = (pJT-alphaT)/(PJ_plus*pJT);
				ans = Math.min(ans, eps2);
			}
		}
		return ans;
	}
	
	private Vector<Integer> getJset(Vector<Integer> t,Vector<Integer> J_minus)
	{
		Vector<Integer> ans = new Vector<Integer> ();
		Iterator<Integer> iter = t.iterator();
		while(iter.hasNext())
		{
			int s = iter.next();
			for(int i = 0; i < edges[s].length; i++)
				if(edges[s][i] && J_minus.contains(i))
					if(!ans.contains(i))
						ans.add(i);
		}
		return ans;
	}
	
	private double getSetPrices(Vector<Integer> set)
	{
		double ans = 0;
		Iterator<Integer> iter = set.iterator();
		while(iter.hasNext())
			ans = ans + prices[iter.next()];
		return ans;
	}
	
	private double getSetAlphas(Vector<Integer> set)
	{
		double ans = 0;
		Iterator<Integer> iter = set.iterator();
		while(iter.hasNext())
			ans = ans + alphas[iter.next()];
		return ans;
	}
	
	private Vector<Vector<Integer>> getSubsets(Vector<Integer> S)
	{
		Vector<Vector<Integer>> subsets = new Vector<Vector<Integer>>();
		subsets.add(S);
		return getSubsetRec(subsets);
		
	}
	
	private Vector<Vector<Integer>> getSubsetRec(Vector<Vector<Integer>> subsets)
	{
		if(subsets.get(0).size() == 0)
			return subsets;
		else
		{
			Vector<Integer> v = subsets.remove(0);
			Integer i = v.remove(0);
			subsets.add(0,v);
			subsets = getSubsetRec(subsets);
			Vector<Vector<Integer>> ans = new Vector<Vector<Integer>>();
			Iterator<Vector<Integer>> iter = subsets.iterator();
			while(iter.hasNext())
			{
				Vector<Integer> v1 = iter.next();
				Vector<Integer> v2 = new Vector<Integer>(v1);
				v1.add(i);
				ans.add(v1);
				ans.add(v2);
			}
			return ans;
		}
	}
	
	private double getPJ( Vector<Integer> vJ)
	{
		double ans = 0;
		Iterator<Integer> iter = vJ.iterator();
		int ind = -1;
		while(iter.hasNext())
		{
			ind = iter.next();
			ans = ans + prices[ind];
		}
		return ans;
	}
	private double[] getAlphas()
	{
		double[] ans = new double[U.length];
		for(int i = 0; i < ans.length; i++)
			ans[i] = 1/(double)ans.length;
		return ans;
	}
	
	private double[] getInitPrices()
	{
		int[] maxA = getMaxArr();
		double[] ans = new double[maxA.length];
		int sum = 0;
		for(int i = 0; i < maxA.length;i++)
		{
			sum = sum + maxA[i];
		}
		//System.out.println(sum);
		for(int j = 0; j < maxA.length;j++)
		{
			//System.out.println(maxA[j]);
			ans[j] = (maxA[j]/(double)sum);
			//System.out.println(ans[j]);
		}
		return ans;
	}
	
	//gets uniform prices
	/*
	private double[] getInitPrices()
	{
	//	int[] maxA = getMaxArr();
		double[] ans = new double[U[0].length];
		
		for(int j = 0; j < U[0].length;j++)
		{
			//System.out.println(maxA[j]);
			ans[j] = 1/(double)U[0].length;
			//System.out.println(ans[j]);
		}
		System.out.println("init prices: ");
		hello.printDoubleArray(ans);
		return ans;
	}
	*/
	
	private int[] getMaxArr()
	{
		int[] ans = new int[U[0].length];
		for(int j = 0; j < U[0].length;j++)
		{
			int max = 0;
			for(int i = 0; i < U.length;i++)
			{
				if(U[i][j] > max)
					max = U[i][j];
			}
			ans[j] = max;
		}
		return ans;
	}
	
	//gets the minimal utility for each item
	/*
	public int[] getMaxArr()
	{
		int[] ans = new int[U[0].length];
		for(int j = 0; j < U[0].length;j++)
		{
			int max = 100;
			for(int i = 0; i < U.length;i++)
			{
				if(U[i][j] < max)
					max = U[i][j];
			}
			ans[j] = max;
		}
		return ans;
	}
	*/
	private static int[][] copyIntSquareArr(int[][] db)
	{
		//System.out.println("db.length - "+ db.length);
		//System.out.println("db[0].length - "+ db[0].length);
		//hello.printBooleanDoubleArray(db);
		int[][] ans = new int[db.length][db[0].length];
		for (int i = 0;i < db.length;i++)
		{
			//System.out.println(i);
			//System.out.println(db[i].length);
			for (int j = 0;j < db[i].length;j++)
				ans[i][j] = db[i][j];
		}
		return ans;
	}
	
	private double[] getMArr(double[] price)
	{
	//	System.out.println("starting getMArr with: ");
	//	hello.printDoubleArray(price);
	//	System.out.println("U:  ");
	//	hello.printDoubleArray(U);
		double[] ans = new double[U.length];
		for(int i = 0; i < U.length;i++)
		{
			double max = 0;
			for(int j = 0; j < U[i].length;j++)
			{
		//		System.out.println("U[i][j]: " + U[i][j]);
				if(U[i][j]/price[j] > max)
					max = U[i][j]/price[j];
			//	System.out.println("max " + max);
			}
			ans[i] = max;
		}
	//	System.out.println("Finishing getMArr with: ");
	//	hello.printDoubleArray(ans);
		return ans;
	}
	
	private double[] getMArr2(double[] price)
	{
		//System.out.println("starting getMArr2 with: ");
		//hello.printDoubleArray(price);
		//System.out.println("U:  ");
		//hello.printDoubleArray(U);
		double[] ans = new double[U.length];
		for(int i = 0; i < U.length;i++)
		{
			double max = 0;
			double max2 = 0;
			for(int j = 0; j < U[i].length;j++)
			{
		//		System.out.println("U[i][j]: " + U[i][j]);
				if(U[i][j]/price[j] > max)
					max = U[i][j]/price[j];
			}
			for(int j = 0; j < U[i].length;j++)
				if(U[i][j]/price[j] > max2 && U[i][j]/price[j] < 0.999999 * max)
						max2 = U[i][j]/price[j];
			//	System.out.println("max " + max);
			
			ans[i] = max2;
		}
		//System.out.println("Finishing getMArr2 with: ");
		//hello.printDoubleArray(ans);
		return ans;
	}
	
	private boolean[][] getEdges(double[] price, double[] M)
	{
		//System.out.println("edges:");
		boolean[][] ans = new boolean[U.length][U[0].length];
		for(int i = 0; i < U.length;i++)
		{
			for(int j = 0; j < U[i].length;j++)
			{
				//System.out.println("U[i][j]/price[j] = " + U[i][j]/price[j]);
				if(Math.abs(U[i][j]/price[j] - M[i]) < 0.0000001)
					ans[i][j] = true;
			}
		}
		//hello.printBooleanDoubleArray(ans);
		//System.out.println("::::::::::::::::::");
		return ans;
	}
	
	private Vector<Pair> getMaxCycle(Vector<Vector<Pair>> cycles)
	{
		Vector<Pair> maxCycle = null;
		Vector<Pair> temp;
		double max = 0;
		Iterator<Vector<Pair>> iter_cycle = cycles.iterator();
		while(iter_cycle.hasNext())
		{
			temp = iter_cycle.next();
			Iterator<Pair> iter_pair = temp.iterator();
			Pair temp_pair1 = temp.lastElement();
			Pair temp_pair2 = temp.get(0);
			double mone = U[temp_pair2.getFirst()][temp_pair1.getSecond()];
			double mechane = U[temp_pair2.getFirst()][temp_pair2.getSecond()];	
			iter_pair.next();
			while(iter_pair.hasNext())
			{
				temp_pair1 = temp_pair2;
				temp_pair2 = iter_pair.next();
				mone = mone * U[temp_pair2.getFirst()][temp_pair1.getSecond()];
				mechane = mechane * U[temp_pair2.getFirst()][temp_pair2.getSecond()];
				if(mone/mechane > max)
				{
				//	System.out.println("updating max cycle  = " + temp); 
				//	System.out.println("updating max cycle with mone  = " + mone + ", mechane = " + mechane); 
					maxCycle = temp;
					max = mone/mechane;
				}
			}
			
		}
		return maxCycle;
	}
	
	private int getRedItem(int ind, Vector<Pair> cycle)
	{
		Iterator<Pair> iter = cycle.iterator();
		while(iter.hasNext())
		{
			Pair p = iter.next();
			if(p.getFirst() == ind)
				return p.getSecond();
		}
		return -1;
	}
	
	private double getMaxRed(int ind, int item, Vector<Pair> cycle)
	{
		//System.out.println("starting getMaxRed on cycle: " + cycle);
		Pair temp_pair1 = cycle.lastElement();
		Pair temp_pair2 = cycle.get(0);
		Iterator<Pair> iter_pair = cycle.iterator();
		double mone = U[temp_pair2.getFirst()][temp_pair1.getSecond()];
		double mechane = U[temp_pair2.getFirst()][temp_pair2.getSecond()];	
		iter_pair.next();
		while(iter_pair.hasNext())
		{
			temp_pair1 = temp_pair2;
			temp_pair2 = iter_pair.next();
			mone = mone * U[temp_pair2.getFirst()][temp_pair1.getSecond()];
			mechane = mechane * U[temp_pair2.getFirst()][temp_pair2.getSecond()];
		}
		
		Iterator<Pair> iter = cycle.iterator();
		while(iter.hasNext())
		{
			Pair p = iter.next();
			if(p.getFirst() == ind && p.getSecond() == item)
			{
				//System.out.println("updating max reduction with mone  = " + mone + ", mechane = " + mechane); 
				mechane = mechane/U[p.getFirst()][p.getSecond()];
				//System.out.println("after dividing mone  = " + mone + ", mechane = " + mechane); 
				double x = mone / mechane;
				return U[p.getFirst()][p.getSecond()] - x;
			}
		}
		return -1;
	}
		
		private double[][] getMaxReductions(Flow_net_2 FN)
		{
			double[][] ans = new double[alphas.length][prices.length];
			for(int i = 0; i < alphas.length; i++)
			{				
				Vector<Vector<Pair>> cyclesContZero = FN.cyclesContaining(i);
				Vector<Pair> maxCycle = getMaxCycle(cyclesContZero);
				//System.out.println("max cycle = " + maxCycle); 
				int redItem = getRedItem(i,maxCycle);
				//System.out.println("reduction item = " + redItem); 
				double maxRed = getMaxRed(i,redItem,maxCycle);
				//System.out.println("max reduction = " + maxRed);
				ans[i][redItem] = maxRed;
			}
			return ans;
		}
		
		private int[] getTypes(Flow_net_2 FN)
		{
			int[] ans = new int[alphas.length];
			double[][] alloc = FN.getAlloc();
			for(int i = 0; i < ans.length; i++)
			{
				int count_allocs = 0;
				for(int j = 0; j < prices.length;j++)
				{
					if(alloc[i][j] > 0.9999999)
						ans[i] = 1;
					else
						if(alloc[i][j] > 0)
							count_allocs++;
				}
				if(ans[i] == 0)
				{
					if(count_allocs > 1)
						ans[i] = 2;
				}
			}
			return ans;	
		}
		
		private boolean canReduce()
		{
			int rand = (int)(Math.random() * U.length);
			for(int i = 0; i < U.length; i++)
			{
				int ind = (rand + i)%U.length;
				if(types[ind] < 2)
				{
					for(int j = 0; j < U[ind].length; j++)
					{
						if(maxReductions[ind][j] > 1)
						{
							U[ind][j] = U[ind][j] - (int)maxReductions[ind][j];
							//System.out.println("reducing " + (int)maxReductions[ind][j] + " for " + ind );
							return true;
						}
					}
				}
			}
			/*
			for(int i = 0; i < U.length; i++)
			{
				int ind = (rand + i)%U.length;
				if(types[ind] == 0)
				{
					for(int j = 0; j < U[ind].length; j++)
					{
						if(maxReductions[ind][j] >= 1)
						{
							U[ind][j] = U[ind][j] - (int)maxReductions[ind][j];
							System.out.println("reducing " + (int)maxReductions[ind][j] + " for " + ind );
							return true;
						}
					}
				}
			}
				*/	
			return false;		
		}
}
